;-------------------------------------------------------------------------------
;	DS2 - ridici jednotka (master)
;
;	18F25K50 / 20MHz			
;
;	09/2016 Milan Srajer
;-------------------------------------------------------------------------------
;
;               
;	                   ---------9V/13V-----------------o
;		           |			           x
;			   |			           o-- +5V
;                          |  --------\/--------           o-- GND
;                          ---|MCLR     PGD/RB7|-----------o
;                       TEP  -|RA0      PGC/RB6|-----------o
;                       BUS  -|RA1      PGM/RB5|-  TL4
;  	      Disp DB7 (BF)  -|RA2          RB4|-  TL3
;                  Disp DB6  -|RA3          RB3|-  TL2
;                  Disp DB4  -|RA4          RB2|-  TL1
;                  Disp DB5  -|RA5      SCL/RB1|-  RTC (DS1307)
;                        0V  -|Vss      SDA/RB0|-  RTC (DS1307)
;                  XT 20MHz  -|OSC1         Vdd|-  +5V
;                  XT 20MHz  -|OSC2/RA6     Vss|-   0V
;                   Disp RS  -|RC0       RX/RC7|-  PIEZOH
;                   Disp RW  -|RC1       TX/RC6|-  PIEZOT
;                   Disp E   -|RC2       D+/RC5|-  USB D+
;                     C USB  -|Vusb      D-/RC4|-  USB D-
;                             ------------------
;   
;
;-------------------------------------------------------------------------------
	LIST P = PIC18F25K50
	#include<p18f25k50.inc>		;header file
;---------------------------------------
	CONFIG MCLRE = ON		;MCLR pin enabled, RE3 input disabled
	CONFIG LVP = OFF		;Single-Supply ICSP disabled
	CONFIG DEBUG = OFF		;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
	CONFIG WDTEN = OFF		;WDT disabled in hardware (SWDTEN ignored)
	CONFIG FOSC = HSH		;HS oscillator, high power 16MHz to 25MHz
	CONFIG CFGPLLEN = OFF		;Oscillator used directly
	CONFIG ICPRT = OFF		;ICPORT disabled
;---------------------------------------
	#include <pic_define.inc>	;definice pinu a registru
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Zacatek behu programu
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
	org	0x00000		;reset vektor
	goto	initial
	org	0x00020
;-------------------------------------------------------------------------------
;	Knihovny
;-------------------------------------------------------------------------------
	#include	<piclib_OLED.inc>	;komunikace s displejem
	#include	<piclib_BCD.inc>	;prace s BCD
	#include	<piclib_EEP.inc>	;ovladani eeprom
	#include	<piclib_BUS.inc>	;komunikace po sbernici
	#include	<piclib_I2C.inc>	;komunikace I2C
	#include	<piclib_DS18B20.inc>	;komunikace s DS18B20
	#include	<piclib_DS1307.inc>	;komunikace s RTC DS1307
;-------------------------------------------------------------------------------
;	Podprogramy
;-------------------------------------------------------------------------------
	#include	<picsub_BUS.inc>	;komunikace po sbernici
	#include	<picsub_RTC.inc>	;zobrazovani data a casu
	#include	<picsub_menu.inc>	;menu
	#include	<picsub_hlaseni.inc>	;zobrazovani textovych informaci a zvuky alarmu
	#include	<picsub_limity.inc>	;vyhodnoceni limitu
;-------------------------------------------------------------------------------
;	Konfigurace procesoru
;-------------------------------------------------------------------------------
initial
	clrf	FSR2H,A		;Access RAM (00h-60h) od 000h GPR
	clrf	FSR2L,A
;       ------------------------
	movlb	0xF		;banka 15, BSR = F, SFR
;       ------------------------
	clrf	ANSELA		;port A digitalni
	clrf	ANSELB		;port B digitalni
	clrf	ANSELC		;port C digitalni
;       ------------------------
	movlb	0x0		;banka 0, BSR = 0, Access RAM
;       ------------------------
	movlw	b'00000000'	;prednastaveni stavu portu
	movwf	PORTA
	movwf	LATA
	movlw	b'00000011'
	movwf	PORTB
	movwf	LATB
	movlw	b'00000000'
	movwf	PORTC
	movwf	LATC
;       ------------------------
	movlw	b'11000011'
	movwf	TRISA		;port A vstupni/vystupni
	movlw	b'11111111'
	movwf	TRISB		;port B vstupni/vystupni
	movlw	b'00111000'
	movwf	TRISC		;port C vstupni/vystupni
;-------------------------------
	bsf	SSP1STAT,SMP	;I2C slew rate control disabled
	movlw	b'00101000'	;I2C master mod, zapni SSP modul, RB0 a RB1 ve funkci SDA, SCL
	movwf	SSP1CON1
	movlw	0x31		;set I2C clock rate to 100kHz (20MHz osc)
	movwf	SSP1ADD
;-------------------------------
	movlw	b'00000111'	;nastaveni citace/casovace TIMER0
;
;	TMR0ON	  0		stop timer
;	T08BIT 	   0		16-bit
;	T0CS   	    0		internal clock source Fosc/4
;	T0SE   	     0		nastupna hrana - zde nema vyznam
;	PSA	      0		prescaler assigned
;	T0PS	       111	prescaler 1:256
;
	movwf	T0CON
;-------------------------------
	movlw	b'00100000'	;nastaveni preruseni
;
;	GIE/GIEH  0		vypni vsechna preruseni
;	PEIE/GIEL  0		vypni vsechna low priority preruseni
;	TMR0IE	    1		zapni preruseni od TMR0
;	INT0IE	     0		vypni INT0 externi preruseni
;	IOCIE	      0		vypni IOCx preruseni pri zmene na portu
;	TMR0IF	       0	nuluj prizak preruseni TMR0
;	INT0IF	        0	nuluj prizak preruseni INT0
;	IOCIF	         0	nuluj prizak preruseni IOCx
;
	movwf	INTCON
;       ------------------------
	bcf	RCON,IPEN	;disable priority levels on interrupts
	bcf	INTCON2,TMR0IP	;low priority of TMR0 oweflow bit
;-------------------------------
	call	beep
;       ------------------------
	call	dispini		;1. inicializace OLED
;       ------------------------
	clrf	SYS		;vymaz systemovy registr
	bsf	DISDYN		;prednastav dynamicke zobrazeni druheho radku
;       ------------------------
	call	c2s
;================================================================================
;================================================================================
;
;	*****	*****	*****	****	*****
;	*	  *	*   *	*   *	  *
;	*****	  *	***** 	****	  *
;	    *	  *	*   *	*  *	  *
;	*****	  *	*   *	*   *	  *
;
;================================================================================
;================================================================================
start	call	zakmit
	call	beep
;	------------------------
	call	dispini		;2. inicializace OLED
	call	dispclr		;smaz displej
;	------------------------
	movlw	0x05
	call	pozice
;	------------------------
	call	znak_S		;uvodni logo
	call	znak_m
	call	znak_a
	call	znak_r
	call	znak_t
	call	znak_h
	call	znak_o
	call	znak_m
	call	znak_e
;	------------------------
	movlw	0x48
	call	pozice
;	------------------------
	call	znak_2
	call	znak_te
	call	znak_0
;-------------------------------
	bcf	SLEEPD		;vymaz pozadavek uspani displeje
	bcf	BEEPD		;vymaz pozadavek pipnuti
	bsf	PRVNI		;nastav priznak prvniho pruchodu
	clrf	TXS		;vymaz registr priznaku
	clrf	ERR		;vymaz chybovy registr
	clrf	RADEK2		;vymaz ukazatel druheho radku
;       ------------------------
	call	loadlim		;nacti limity z eeprom
	call	t_mer		;zmer teplotu internim cidlem
	movlw	.20		;prodleva sporice, nasobek cteni ze sbernice (20 x 4 = 80s)
	movwf	SPO
;	------------------------
	call	c2s		;pockej 2 sekundy
;	------------------------
	call	bus01		;obsluz vsechny periferie na sbernici
;--------------------------------------------------------------------------------
navrat				;navratovy vektor z menu
	call	zakmit
	call	dispclr
;================================================================================
;
;	Rutinni zalezitost
;
;================================================================================
;	Rutina pri zapnutem displeji
;--------------------------------------------------------------------------------
rutina
	call	t_cti		;cti teplotu z interniho cidla
	call	t_mer		;proved nove mereni internim cidlem
	call	limity		;vyhodnot limity vcetne alarmu
	call	hlaseni		;zobraz chybova hlaseni
	call	obndisp		;zobraz hodnoty na displeji vcetne limitu a pipani
;-------------------------------
restrut
	movlw	.200		;kontrola tlacitek po dobu cca 4s
        movwf	CEK3
rut13	movlw	.100
	movwf	CEK2
rut12	movlw	.100
	movwf	CEK1
;       ------------------------
rut11	btfss	TL1
	goto	menu1_1		;menu displeje
	btfss	TL2
	goto	dalsi_polozka	;dalsi polozka spodniho radku
	btfss	TL3
	goto	rest		;priprav restart
	btfss	TL4
	goto	menu2_1		;menu nastaveni limitu
;       ------------------------
	decfsz	CEK1,1
        goto	rut11
        decfsz	CEK2,1
        goto	rut12
        decfsz	CEK3,1
        goto	rut13
;-------------------------------
	call	rtc_cteni	;zjisti aktualni datum a cas z RTC
	call	bus00		;obsluz vsechny periferie na sbernici
;       ------------------------
	decfsz	SPO,F		;pokud je cas, spust sporic displeje
	goto	rut14
	call	sporic
;       ------------------------
rut14	goto	dis_sta		;zjisti, zda neni treba vypnout displej
;       ------------------------
	goto	rutina
;--------------------------------------------------------------------------------
;	Rutina pri vypnutem displeji
;--------------------------------------------------------------------------------
rutina2
	call	t_cti		;cti teplotu z interniho cidla
	call	t_mer		;proved nove mereni internim cidlem
	call	limity		;vyhodnot limity vcetne alarmu
;       ------------------------
	movf	ALARM,F		;zapni dipslej, pokud je alarm
	btfss	STATUS,Z
	goto	start
;-------------------------------
	movlw	.200		;kontrola tlacitek po dobu 4s
        movwf	CEK3
rut23	movlw	.100
	movwf	CEK2
rut22	movlw	.100
	movwf	CEK1
;       ------------------------
rut21	btfss	TL1		;pri stisku jakehokoliv tlacitka zapni displej
	goto	start
	btfss	TL2
	goto	start
	btfss	TL3
	goto	start
	btfss	TL4
	goto	start
;       ------------------------
	decfsz	CEK1,1
        goto	rut21
        decfsz	CEK2,1
        goto	rut22
        decfsz	CEK3,1
        goto	rut23
;-------------------------------
	call	rtc_cteni	;zjisti aktualni datum a cas z RTC
	call	bus00		;obsluz vsechny periferie na sbernici
;       ------------------------
	goto	rutina2
;================================================================================
;
;	Restart zarizeni = TL3 + TL4
;
;================================================================================
rest
	btfsc	TL3		;pokud bylo uvolneno TL3
	goto	restrut		;vrat se na rutinu
	btfss	TL4		;pokud bylo stisknuto jeste TL4
	goto	initial		;resetuj vsechno
	goto	rest
;================================================================================
;
;	Obnoveni pohotovostniho displeje vcetne zobrazeni limitu a pipnuti pri
;	prekroceni limitu
;
;================================================================================
;	Zobrazeni teplot (TxDE TxJE , TxDT )
;-------------------------------
obndisp
	call	zakmit
;-------------------------------
;	Zobrazi teplotu kotle
;-------------------------------
tepkot
	movlw	00h		;zobraz T2 - teplotu kotle
	call	pozice
;	------------------------
	movf	T2DE,W
	call	disphex
	movf	T2JE,W
	call	disphex
	call	znak_te
	movf	T2DT,W
	call	disphex
	call	znak_st
;-------------------------------
;	Zobrazi teplotu vody
;-------------------------------
tepvod
	movlw	08h		;zobraz T4 - teplotu vody
	call	pozice
;	------------------------
	movf	T4DE,W
	call	disphex
	movf	T4JE,W
	call	disphex
	call	znak_te
	movf	T4DT,W
	call	disphex
	call	znak_st
;--------------------------------------------------------------------------------
;	Zobrazi stav jimky
;-------------------------------
zobrJ
	movlw	0Fh
	call	pozice
;	------------------------
	btfsc	JIM,7		;zobrazeni chyby sondy (E pred hodnotou)
	call	znak_E
	btfss	JIM,7
	call	znak_
;	------------------------
	btfsc	JIM,0
	goto	j25
	btfsc	JIM,1
	goto	j50
	btfsc	JIM,2
	goto	j75
	btfsc	JIM,3
	goto	j100
	btfsc	JIM,4
	goto	j125
;	------------------------
j0	call	znak_
	call	znak_
	call	znak_0
	call	znak_pr
	goto	zoblim
;	------------------------
j25	call	znak_
	call	znak_2
	call	znak_5
	call	znak_pr
	goto	zoblim
;	------------------------
j50	call	znak_
	call	znak_5
	call	znak_0
	call	znak_pr
	goto	zoblim
;	------------------------
j75	call	znak_
	call	znak_7
	call	znak_5
	call	znak_pr
	goto	zoblim
;	------------------------
j100	call	znak_1
	call	znak_0
	call	znak_0
	call	znak_pr
	goto	zoblim
;	------------------------
j125	call	znak_1
	call	znak_2
	call	znak_5
	call	znak_pr
	goto	zoblim
;--------------------------------------------------------------------------------
;	Zobrazi priznaky limitu
;--------------------------------------------------------------------------------
zoblim
	movlw	0x05		;pozice priznaku T2
	call	pozice
;	------------------------
	call	znak_		;smaz priznak
;	------------------------
	movlw	0x05		;znovu pozice
	call	pozice
;	------------------------
	btfsc	T2S1		;jestli je pod limit 1
	call	znak_sd		;zobraz sipku dolu
	btfsc	T2S2		;jestli je nad limit 2
	call	znak_sn		;zobraz sipku nahoru
;-------------------------------
	movlw	0x0D		;pozice priznaku T4
	call	pozice
;	------------------------
	call	znak_		;smaz priznak
;	------------------------
	movlw	0x0C		;znovu pozice
	call	pozice
;	------------------------
	btfsc	T4S1		;jestli je pod limit 1
	call	znak_sd		;zobraz sipku dolu
	btfsc	T4S2		;jestli je nad limit 2
	call	znak_sn		;zobraz sipku nahoru
;--------------------------------------------------------------------------------
;	Pipne pri prekroceni limitu (pozadavek BEEPD = 1)
;--------------------------------------------------------------------------------
	btfsc	BEEPD
	call	beep_d
;	------------------------
	bcf	PRVNI		;zrus priznak prvniho pruchodu
;--------------------------------------------------------------------------------
;
;	Zobrazeni udaju na spodnim radku
;
;--------------------------------------------------------------------------------
	btfsc	DISDC
	goto	datum_a_cas
;-------------------------------
	movf	RADEK2,W	;nacti ukazatel zobrazeni
	sublw	0x00		;je-li RADEK2 0..
	btfsc	STATUS,Z
	call	zobr_T5		;zobraz teplotu doma

	movf	RADEK2,W	;nacti ukazatel zobrazeni
	sublw	0x01		;je-li RADEK2 1..
	btfsc	STATUS,Z
	call	zobr_T1		;zobraz vstup kotle

	movf	RADEK2,W	;nacti ukazatel zobrazeni
	sublw	0x02		;je-li RADEK2 2..
	btfsc	STATUS,Z
	call	zobr_T3		;zobraz vstup bojleru

	movf	RADEK2,W	;nacti ukazatel zobrazeni
	sublw	0x03		;je-li RADEK2 3..
	btfsc	STATUS,Z
	call	zobr_napeti_aku	;zobraz napeti akumulatoru
;-------------------------------
;	Inkrementace ukazatele spodniho radku
;-------------------------------
	btfsc	DISDYN
	incf	RADEK2,F
;-------------------------------
;	Nulovani ukazatele po posledni polozce
;-------------------------------
	movf	RADEK2,W	;nacti ukazatel zobrazeni
	sublw	0x04		;je-li RADEK2 4..
	btfsc	STATUS,Z
	clrf	RADEK2
;	------------------------
	return			;obndisp
;--------------------------------------------------------------------------------
;	Zobrazeni napeti akumulatoru (ADE AJE , ADT V)
;-------------------------------
zobr_napeti_aku
	movlw	40h
	call	pozice
;	------------------------
	call	znak_n
	call	znak_a
	call	znak_p
	call	znak_eh
	call	znak_t
	call	znak_ic
	call	znak_
	call	znak_b
	call	znak_a
	call	znak_t
	call	znak_e
	call	znak_r
	call	znak_i
	call	znak_e
	call	znak_
;	------------------------
	movf	ADE,W		;vezmi desitky
	btfss	STATUS,Z	;jestli nejsou 0..
	call	disphex		;zobraz je
;	------------------------
	movf	ADE,W		;vezmi znovu desitky
	btfsc	STATUS,Z	;jestli jsou 0..
	call	znak_		;zobraz prazdny znak
;	------------------------
	movf	AJE,W		;vezmi jednotky
	call	disphex		;zobraz je
	call	znak_te		;.
	movf	ADT,W		;vezmi desetiny
	call	disphex		;zobraz je
	call	znak_V		;V
;	------------------------
	return
;--------------------------------------------------------------------------------
;	Zobrazeni teploty na vstupu do kotle
;-------------------------------
zobr_T1
	movlw	40h
	call	pozice
;	------------------------
	call	znak_v
	call	znak_s
	call	znak_t
	call	znak_u
	call	znak_p
	call	znak_
	call	znak_d
	call	znak_o
	call	znak_
	call	znak_k
	call	znak_o
	call	znak_t
	call	znak_l
	call	znak_e
	call	znak_
;	------------------------
	movf	T1DE,W
	call	disphex
	movf	T1JE,W
	call	disphex
	call	znak_te
	movf	T1DT,W
	call	disphex
	call	znak_st
;	------------------------
	call	znak_
;	------------------------
	return
;--------------------------------------------------------------------------------
;	Zobrazeni teploty na vstupu bojleru
;-------------------------------
zobr_T3
	movlw	40h
	call	pozice
;	------------------------
	call	znak_p
	call	znak_rh
	call	znak_ic
	call	znak_v
	call	znak_o
	call	znak_d
	call	znak_
	call	znak_b
	call	znak_o
	call	znak_j
	call	znak_l
	call	znak_e
	call	znak_r
	call	znak_u
	call	znak_
;	------------------------
	movf	T3DE,W
	call	disphex
	movf	T3JE,W
	call	disphex
	call	znak_te
	movf	T3DT,W
	call	disphex
	call	znak_st
;	------------------------
	return
;--------------------------------------------------------------------------------
;	Zobrazeni teploty v obyvaku
;-------------------------------
zobr_T5
	movlw	40h
	call	pozice
;	------------------------
	call	znak_t
	call	znak_e
	call	znak_p
	call	znak_te
	call	znak_
	call	znak_v
	call	znak_
	call	znak_o
	call	znak_b
	call	znak_yc
	call	znak_v
	call	znak_ac
	call	znak_k
	call	znak_u
	call	znak_
;	------------------------
	movf	TDE,W
	call	disphex
	movf	TJE,W
	call	disphex
	call	znak_te
	movf	TDT,W
	call	disphex
	call	znak_st
;	------------------------
	return
;--------------------------------------------------------------------------------
;	Zobrazeni data a casu	_H:MM _D.M.20RR
;-------------------------------
datum_a_cas
	call	rtc_cteni
;	------------------------
	movlw	0x42
	call	pozice
;	------------------------
	call	cas_h
	call	znak_dt
	call	cas_mm
	call	znak_
	call	dat_d
	call	znak_te
	call	dat_m
	call	znak_te
	call	dat_4r
;	------------------------
	return
;--------------------------------------------------------------------------------
dalsi_polozka
	call	zakmit
;	------------------------
	btfsc	DISSTA
	incf	RADEK2,F	;inkrementace ukazatele spodniho radku
;	------------------------
	movf	RADEK2,W	;vynuluje ukazatel, pokud je pres limit
	sublw	0x04
	btfsc	STATUS,Z
	clrf	RADEK2
;	------------------------
	goto	rutina
;================================================================================
;
;	Vypinani displeje pri vypadku site nebo aktivaci alarmu
;
;================================================================================
dis_sta
	btfsc	PG1		;doslo k zakodovani?
	goto	dis_vy1		;..vypni displej
	btfsc	PG3		;doslo k vypadku site?
	goto	dis_vy2		;..vypni displej
	goto	rutina		;pokud nic z toho, vrat se
;-------------------------------
dis_vy1	call	dispclr
;	------------------------
	movlw	0x02
	call	pozice
;	------------------------
	call	znak_a
	call	znak_l
	call	znak_a
	call	znak_r
	call	znak_m
	call	znak_
	call	znak_a
	call	znak_k
	call	znak_t
	call	znak_i
	call	znak_v
	call	znak_o
	call	znak_v
	call	znak_ac
	call	znak_n
	call	znak_vy
;	------------------------
	goto	dis_vyp
;-------------------------------
dis_vy2	call	dispclr
;	------------------------
	movlw	0x03
	call	pozice
;	------------------------
	call	znak_V
	call	znak_yc
	call	znak_p
	call	znak_a
	call	znak_d
	call	znak_e
	call	znak_k
	call	znak_
	call	znak_s
	call	znak_ic
	call	znak_t
	call	znak_eh
	call	znak_vy
;	------------------------
	goto	dis_vyp
;-------------------------------
dis_vyp	call	beep
	call	c2s
	call	beep
;	------------------------
	call	dispclr
;       ------------------------
	bsf	SLEEPD		;nastav priznak uspani displeje
	movlw	b'00001000'	;vypni displej
	call	dis_ini
	goto	rutina2		;a skoc na rutinu2

;================================================================================
;
;	Sporic displeje
;
;================================================================================
sporic
	call	dispclr		;smaz displej
;	------------------------
	movlw	b'00000111'	;Entry mode, shift display to the left
	call	dis_ini
;	------------------------
	movlw	14h		;21. misto 1. radku
	call	pozice
;	------------------------
	call	znak_S
	call	cs1
	call	znak_m
	call	cs1
	call	znak_a
	call	cs1
	call	znak_r
	call	cs1
	call	znak_t
	call	cs1
	call	znak_h
	call	cs1
	call	znak_o
	call	cs1
	call	znak_m
	call	cs1
	call	znak_e
	call	cs1
	call	znak_
	call	cs1
	call	znak_2
	call	cs1
	call	znak_te
	call	cs1
	call	znak_0
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
	call	znak_
	call	cs1
;	------------------------
	movlw	b'00000110'	;Entry mode, no shift display
	call	dis_ini
;	------------------------
	movlw	b'00000010'	;DDRAM - return home
	call	dis_ini
;	------------------------
	call	dispclr		;smaz displej
;	------------------------
	movlw	.30		;dalsi sporic za minutu
	movwf	SPO
;	------------------------
	return
;================================================================================
;
;	Zobrazeni BCD na displeji
;
;================================================================================
zoBCD
	movf	BCD,W
	call	disphe2
;	------------------------
	return
;================================================================================
;
;	Nacteni limitu z eeprom
;
;================================================================================
loadlim
	eeread	0x00
	movwf	T2L1
;       ------------------------
	eeread	0x01
	movwf	T2L2
;       ------------------------
	eeread	0x02
	movwf	T4L1
;       ------------------------
	eeread	0x03
	movwf	T4L2
;       ------------------------
	return
;================================================================================
;
;	Zvuky
;
;================================================================================
beep
	movlw	.5		;beep kratce (4kHz)
        movwf	CEK3
bee04	movlw	.60
        movwf	CEK2
bee03	movlw	.200
        movwf	CEK1
bee01	decfsz	CEK1,1
        goto	bee01
	bsf	PIEZOT
	movlw	.200
        movwf	CEK1
bee02	decfsz	CEK1,1
        goto	bee02
	bcf	PIEZOT	
        decfsz	CEK2,1
        goto	bee03
        decfsz	CEK3,1
        goto	bee04
;	------------------------
	return
;--------------------------------------------------------------------------------
beep_d
	btfsc	PRVNI		;je-li prvni pruchod, nepipej
	return
;	------------------------
	movlw	.25		;beep dlouze (4kHz)
        movwf	CEK3
beed04  movlw	.60
        movwf	CEK2
beed03	movlw	.200
        movwf	CEK1
beed01	decfsz	CEK1,1
        goto	beed01
	bsf	PIEZOT
	movlw	.200
        movwf	CEK1
beed02	decfsz	CEK1,1
        goto	beed02
	bcf	PIEZOT
        decfsz	CEK2,1
        goto	beed03
        decfsz	CEK3,1
        goto	beed04
;	------------------------
	return
;--------------------------------------------------------------------------------
beep_a
	call	beep_h		;pipani pri alarmu
	call	cs5
	call	beep_h
	call	cs5
	call	beep_h
;	------------------------
	return
;-------------------------------
beep_h
	movlw	.25		;beep dlouze hlasite (4kHz)
        movwf	CEK3
beeh04 movlw	.60
        movwf	CEK2
beeh03	movlw	.200
        movwf	CEK1
beeh01	decfsz	CEK1,1
        goto	beeh01
	bsf	PIEZOH
	movlw	.200
        movwf	CEK1
beeh02	decfsz	CEK1,1
        goto	beeh02
	bcf	PIEZOH	
        decfsz	CEK2,1
        goto	beeh03
        decfsz	CEK3,1
        goto	beeh04
;	------------------------
	return
;================================================================================
;
;	Cekaci smycky
;
;================================================================================
zakmit
	movlw	.250		;(15ms) osetreni zakmitu vsech tlacitek
        movwf	CEK2
zak02   movlw	.250
        movwf	CEK1
zak01	decfsz	CEK1,1
        goto	zak01
        decfsz	CEK2,1
        goto	zak02
;       ------------------------
	btfss	TL1
	goto	zakmit
	btfss	TL2
	goto	zakmit
	btfss	TL3
	goto	zakmit
	btfss	TL4
	goto	zakmit
;       ------------------------
	return
;--------------------------------------------------------------------------------
cs1
	movlw	.100		;cekej 0,1s
        movwf	CEK3
cs103   movlw	.207
        movwf	CEK2
cs102	movlw	.5
        movwf	CEK1
cs101	decfsz	CEK1,1
        goto	cs101
        decfsz	CEK2,1
        goto	cs102
        decfsz	CEK3,1
        goto	cs103
;	------------------------
	return
;--------------------------------------------------------------------------------
cs5
	movlw	.40		;cekej 0,5s
        movwf	CEK3
cs503   movlw	.126
        movwf	CEK2
cs502	movlw	.164
        movwf	CEK1
cs501	decfsz	CEK1,1
        goto	cs501
        decfsz	CEK2,1
        goto	cs502
        decfsz	CEK3,1
        goto	cs503
;	------------------------
	return
;--------------------------------------------------------------------------------
c2s
	movlw	.160		;cekej 2s
        movwf	CEK3
c2s03   movlw	.126
        movwf	CEK2
c2s02	movlw	.164
        movwf	CEK1
c2s01	decfsz	CEK1,1
        bra	c2s01
        decfsz	CEK2,1
        bra	c2s02
        decfsz	CEK3,1
        bra	c2s03
;	------------------------
	return
;-------------------------------------------------------------------------------
	end