;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Knihovna podprogramu pro praci s BCD
;
;	Milan rajer, www.milansrajer.cz
;	09/2015
;
;-------------------------------------------------------------------------------
;
;	Volani
;	------------------------
;	call	incBCD		- inkrementuje BCD
;	call	decBCD		- dekrementuje BCD
;	call	roBCD		- rozdeli BCD do BCDD a BCDJ
;	call	slBCD		- slouci BCDD a BCDJ do BCD
;
;
;	Pozadavky - registry
;	------------------------
;	BCD
;	BCDD
;	BCDJ
;
;	BCDMAX - maximalni povolena hodnota v BCD formatu (00-99)
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Inkrementace/dekrementace BCD - prijima a vraci BCD
;
;	BCDMAX	- maximalni hodnota
;
;-------------------------------------------------------------------------------
incBCD	call	roBCD		;rozdeli BCD na desitky = BCDD a jednotky = BCDJ		
;	------------------------
	movf	BCDJ,W		;nacti jednotky
	sublw	0x09		;je-li BCDJ 9
	btfsc	STATUS,Z
	goto	inc_des		;..inkrementuj desitky
	incf	BCDJ,F		;jinak inkrementuj jen jednotky
	goto	inc_lim		;jdi na kontrolu limitu
;	------------------------
inc_des	incf	BCDD,F		;inkrementuj desitky
	clrf	BCDJ,F		;preklop jednotky do 0
;-------------------------------
; Kontrola limitu
;-------------------------------
inc_lim	call	slBCD		;sluc BCD (W = BCD = BCDD + BCDJ)
	subwf	BCDMAX,W	;BCDMAX(F) - BCD(W)
	btfss	STATUS,C
	clrf	BCD		;BCD -> 00
;-------------------------------
	return
;--------------------------------------------------------------------------------
decBCD	call	roBCD		;rozdeli W na desitky = BCDD a jednotky = BCDJ		
;	------------------------
	movf	BCDJ,W		;nacti jednotky
	sublw	0x00		;je-li BCDJ 0..
	btfsc	STATUS,Z
	goto	dec_des		;..dekrementuj desitky
	decf	BCDJ,F		;jinak dekrementuj jen jednotky
	call	slBCD		;sluc BCD
	return
;	------------------------
dec_des	movf	BCDD,W		;nacti desitky
	sublw	0x00		;je-li BCDD 0..
	btfsc	STATUS,Z
	goto	dec_lim		;..nastav BCD na limit BCDMAX
	decf	BCDD,F		;jinak dekrementuj desitky
	movlw	0x09		;preklop jednotky na 9
	movwf	BCDJ
	call	slBCD		;sluc BCD
	return
;	------------------------
dec_lim	movff	BCDMAX,BCD	;BCDMAX -> BCD
	return
;--------------------------------------------------------------------------------
;	Rozdeli BCD na desitky (BCDD) a jednotky (BCDJ), W = 0Fh
;-------------------------------
roBCD	movf	BCD,0
	movwf	BCDD		;desitky
	movwf	BCDJ		;jednotky
;	------------------------
	swapf	BCDD,1
	movlw	0x0F
	andwf	BCDD,1
	andwf	BCDJ,1
;	------------------------
	return
;--------------------------------------------------------------------------------
;	Slouci BCDD a BCDJ do BCD, W=BCD, BCDD a BCDJ zustavaji
;-------------------------------
slBCD	swapf	BCDD,0
	iorwf	BCDJ,0
	movwf	BCD
;	------------------------
	return
;--------------------------------------------------------------------------------