;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Knihovna RTC DS1307
;
;	Procesor:	18F25K50
;	Rychlost:	20MHz
;
;	Milan rajer		
;	08/2016
;
;-------------------------------------------------------------------------------
;
;	Volani
;	------------------------
;	call	rtc_init	- automaticka inicializace
;	call	rtc_cteni	- nacteni casu a data z RTC
;	call	rtc_zapis	- zapsani casu a data do RTC
;
;-------------------------------------------------------------------------------
;
;	Pozadavky - knihovny
;	------------------------
;	piclib_I2C
;
;
;	Pozadavky - registry
;	------------------------
;	SEKUNDY	- pomocne registry
;	MINUTY
;	HODINY
;	DENVTY
;	DEN
;	MESIC
;	ROK
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; Inicializace RTC
;-------------------------------------------------------------------------------
rtc_init
	call	i2c_start	;start komunikace
;       ------------------------
	movlw	b'11010000'	;nacti adresu RTC pro zapis
	call	i2c_posli_byte	;adresuj RTC
	movlw	0x00		;register pointer 00h (sekundy)
	call	i2c_posli_byte
	movlw	0x00		;sekundy=00, CH=0
	call	i2c_posli_byte
;       ------------------------
	call	i2c_stop	;konec komunikace
;       ------------------------
	return
;-------------------------------------------------------------------------------
; Nacte cas a datum z RTC
;-------------------------------------------------------------------------------
rtc_cteni
	call	i2c_start	;start komunikace
;       ------------------------
	movlw	b'11010000'	;nacti adresu RTC pro zapis
	call	i2c_posli_byte	;adresuj RTC
	movlw	0x00		;register pointer 00h (sekundy)
	call	i2c_posli_byte
;       ------------------------
	call	i2c_restart	;restart komunikace
	movlw	b'11010001'	;nacti adresu RTC pro cteni
	call	i2c_posli_byte	;adresuj RTC
;       ------------------------
	call	i2c_cti_byte	;prijmi sekundy
	movff	SSP1BUF,SEKUNDY
	call	i2c_ack		;vysli ACK
	call	i2c_cti_byte	;prijmi minuty
	movff	SSP1BUF,MINUTY
	call	i2c_ack		;vysli ACK
	call	i2c_cti_byte	;prijmi hodiny
	call	i2c_ack		;vysli ACK
	movff	SSP1BUF,HODINY
	call	i2c_cti_byte	;prijmi den v tydnu
	call	i2c_ack		;vysli ACK
	movff	SSP1BUF,DENVTY
	call	i2c_cti_byte	;prijmi den
	call	i2c_ack		;vysli ACK
	movff	SSP1BUF,DEN
	call	i2c_cti_byte	;prijmi mesic
	call	i2c_ack		;vysli ACK
	movff	SSP1BUF,MESIC
	call	i2c_cti_byte	;prijmi rok
	call	i2c_nack	;vysli NACK
	movff	SSP1BUF,ROK
;	------------------------
	call	i2c_stop	;konec komunikace
;	------------------------
	return
;--------------------------------------------------------------------------------
; Zapise cas a datum do RTC
;-------------------------------
rtc_zapis
	call	i2c_start	;start komunikace
;       ------------------------
	movlw	b'11010000'	;nacti adresu RTC pro zapis
	call	i2c_posli_byte	;adresuj RTC
	movlw	0x00		;register pointer 00h (sekundy)
	call	i2c_posli_byte
	movlw	0x00		;sekundy
	call	i2c_posli_byte
	movf	MINUTY,W	;minuty
	call	i2c_posli_byte
	movf	HODINY,W	;hodiny
	call	i2c_posli_byte
	movlw	0x00		;den v tydnu
	call	i2c_posli_byte
	movf	DEN,W		;den
	call	i2c_posli_byte
	movf	MESIC,W		;mesic
	call	i2c_posli_byte
	movf	ROK,W		;rok
	call	i2c_posli_byte
;       ------------------------
	call	i2c_stop	;konec komunikace
;       ------------------------
	return
;--------------------------------------------------------------------------------




