;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Knihovna pro 4-bitove ovladani OLED Winstar 20x4
;	s radicem WS0010 se ctenim busy flag (BF)
;
;	Procesor:	16F876A
;	Rychlost:	20MHz
;
;	Autor:		Milan rajer, www.milansrajer.cz		
;	Datum:		09/2015
;
;	Znakova sada:	Wester-European
;
;-------------------------------------------------------------------------------
;
;	Volani
;	------------------------
;	call	dispini		- inicializace displeje
;	call	dispCZ		- nauci displej ceske znaky
;	call	dispclr		- smazani displeje
;	call	pozice		- nastaveni pozice kurzoru na adresu ulozenou ve W
;	call	znak_X		- zobrazeni znaku X dle znakove sady
;	call	disphex		- zobrazeni hex hodnoty registru W - 1 znak (bity 0-3)	
;	call	disphe2		- zobrazeni hex hodnoty registru W - 2 znaky	
;
;-------------------------------------------------------------------------------
;
;	Pozadavky - registry
;	------------------------
;	TMP1	- pomocne registry
;	TMP2
;	TMP3
;
;	CEK1	- registry cekacich smycek
;	CEK2
;	CEK3
;
;
;	Pozadavky - piny, bity
;	------------------------
;	RS	- konfiguracni piny displeje
;	RW
;	E
;
;	DB4	- datove piny displeje
;	DB5
;	DB6
;	DB7	= busy flag
;
;	DB4_T	- ekvivalenty datovych pinu v registru TRIS (pro nastaveni V/V)
;	DB5_T
;	DB6_T
;	DB7_T
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;	Inicializace
;-------------------------------
dispini	call	d_cs5		;cekani 500ms na vnitrni inicializaci po zapnuti
;       ------------------------
	movlw	b'00000000'	;synchronizace pro 4-bitovou komunikaci
	call	dis_in1
	movlw	b'00000000'
	call	dis_in1
	movlw	b'00000000'
	call	dis_in1
	movlw	b'00000000'
	call	dis_in1
	movlw	b'00000000'
	call	dis_in1
;       ------------------------
	movlw	b'00100000'	;4 bity
	call	dis_in1
;       ------------------------
	movlw	b'00100000'	;4 bity, 2 radky, 5x8 pixelu, west-eu znakova sada
	call	dis_in1
;       ------------------------
	movlw	b'10010000'
	call	dis_in1
;       ------------------------;od teto chvile kontroluje BF
;       ------------------------
	movlw	b'00101001'	;4 bity, 2 radky, 5x8 pixelu, west-eu znakova sada
	call	dis_ini
;       ------------------------
	movlw	b'00001000'	;vypni displej
	call	dis_ini
;       ------------------------
	movlw	b'00000001'	;smaz displej (DDRAM data)
	call	dis_ini
;       ------------------------
	movlw	b'00010100'	;automaticke posouvani kurzoru doprava, displej neposouva 
	call	dis_ini
;       ------------------------
	movlw	b'00010111'	;alfanumericky rezim, vnitrni budice zapnout
	call	dis_ini
;       ------------------------	
	movlw	b'00000010'	;kurzor a displej domu (DDRAM adresa = 00h)
	call	dis_ini
;       ------------------------
	movlw	b'00001100'	;zapni displej, vypni kurzor a neblikej s nim
	call	dis_ini
;       ------------------------
	movlw	b'00000110'	;(entry mode) kurzor doprava, neposouvej displej
	call	dis_ini
;-------------------------------
;	Ceske znaky do displeje
;-------------------------------;
	movlw	b'01000000'	;adresa CGRAM
	call	dis_ini
	movlw	b'00001010'	;data do CGRAM
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00010110'
	call	dis_zna
	movlw	b'00011000'
	call	dis_zna
	movlw	b'00010000'
	call	dis_zna
	movlw	b'00010000'
	call	dis_zna
	movlw	b'00010000'
	call	dis_zna
	movlw	b'00000000'
	call	dis_zna
;-------------------------------;
	movlw	b'01001000'	;adresa CGRAM
	call	dis_ini
	movlw	b'00000010'	;data do CGRAM
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00010001'
	call	dis_zna
	movlw	b'00010001'
	call	dis_zna
	movlw	b'00001111'
	call	dis_zna
	movlw	b'00000001'
	call	dis_zna
	movlw	b'00001110'
	call	dis_zna
	movlw	b'00000000'
	call	dis_zna
;-------------------------------;
	movlw	b'01010000'	;adresa CGRAM
	call	dis_ini
	movlw	b'00001010'	;data do CGRAM
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00001110'
	call	dis_zna
	movlw	b'00010001'
	call	dis_zna
	movlw	b'00011111'
	call	dis_zna
	movlw	b'00010000'
	call	dis_zna
	movlw	b'00001110'
	call	dis_zna
	movlw	b'00000000'
	call	dis_zna
;-------------------------------;
	movlw	b'01011000'	;adresa CGRAM
	call	dis_ini
	movlw	b'00001010'	;data do CGRAM
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00001110'
	call	dis_zna
	movlw	b'00010000'
	call	dis_zna
	movlw	b'00001110'
	call	dis_zna
	movlw	b'00000001'
	call	dis_zna
	movlw	b'00011110'
	call	dis_zna
	movlw	b'00000000'
	call	dis_zna
;-------------------------------;
	movlw	b'01100000'	;adresa CGRAM
	call	dis_ini
	movlw	b'00001010'	;data do CGRAM
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00011111'
	call	dis_zna
	movlw	b'00000010'
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00001000'
	call	dis_zna
	movlw	b'00011111'
	call	dis_zna
	movlw	b'00000000'
	call	dis_zna
;-------------------------------;
	movlw	b'01101000'	;adresa CGRAM
	call	dis_ini
	movlw	b'00000110'	;data do CGRAM
	call	dis_zna
	movlw	b'00000110'
	call	dis_zna
	movlw	b'00010001'
	call	dis_zna
	movlw	b'00010001'
	call	dis_zna
	movlw	b'00010001'
	call	dis_zna
	movlw	b'00010011'
	call	dis_zna
	movlw	b'00001101'
	call	dis_zna
	movlw	b'00000000'
	call	dis_zna
;-------------------------------;
	movlw	b'01110000'	;adresa CGRAM
	call	dis_ini
	movlw	b'00001010'	;data do CGRAM
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00001110'
	call	dis_zna
	movlw	b'00010000'
	call	dis_zna
	movlw	b'00010000'
	call	dis_zna
	movlw	b'00010001'
	call	dis_zna
	movlw	b'00001110'
	call	dis_zna
	movlw	b'00000000'
	call	dis_zna
;-------------------------------;c
	movlw	b'01111000'	;adresa CGRAM
	call	dis_ini
	movlw	b'00001000'	;data do CGRAM
	call	dis_zna
	movlw	b'00010100'
	call	dis_zna
	movlw	b'00001000'
	call	dis_zna
	movlw	b'00000011'
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00000100'
	call	dis_zna
	movlw	b'00000011'
	call	dis_zna
	movlw	b'00000000'
	call	dis_zna
;-------------------------------
	return
;-------------------------------------------------------------------------------
;	Smazani obsahu
;-------------------------------
dispclr
	movlw	b'00000001'
	call	dis_ini
;       ------------------------
	return
;-------------------------------------------------------------------------------
;	Nastaveni pozice kurzoru (W) (BF)
;-------------------------------
pozice
	movwf	TMP1		;priprav data do TMP1
	bsf	TMP1,7		;bit7=1 - budeme zapisovat adresu DDRAM displeje
;       ------------------------
	call	dis_bf		;pockej na uvolneni od displeje
;       ------------------------
	call	dis_wr1		;odesli bity 4-7
	call	dis_wr2		;odesli bity 0-3
;       ------------------------
	return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Vnitrni rutiny
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;	Zapis znaku na pozici kurzoru (W) (BF)
;-------------------------------
dis_zna
	movwf	TMP1		;priprav data do TMP1
;       ------------------------
	call	dis_bf		;pockej na uvolneni od displeje
;       ------------------------
	bsf	RS		;RS=1 - budeme zapisovat data do DDRAM displeje
	call	dis_wr1		;odesli bity 4-7
	call	dis_wr2		;odesli bity 0-3
	bcf	RS		;RS=0
;       ------------------------
	return
;-------------------------------------------------------------------------------
;	Odeslani inicializacniho slova (W) (BF)
;-------------------------------
dis_ini
	movwf	TMP1		;priprav data do TMP1
;       ------------------------
	call	dis_bf		;pockej na uvolneni od displeje
;       ------------------------
	call	dis_wr1		;odesli bity 4-7
	call	dis_wr2		;odesli bity 0-3
;       ------------------------
	return
;-------------------------------------------------------------------------------
;	Odeslani pulky inicializacniho slova (W 4-7)
;-------------------------------
dis_in1
	movwf	TMP1		;priprav data do TMP1
;       ------------------------
	call	dis_wr1		;odesli bity 4-7
;       ------------------------
	call	d_c5m		;pockej 5ms
;       ------------------------
	return
;-------------------------------------------------------------------------------
;	Pripravi bity 4-7 a posle na displej (TMP1)
;-------------------------------
dis_wr1
	bcf	DB7		;vsechny piny do log 0
	bcf	DB6
	bcf	DB5
	bcf	DB4
;       ------------------------
	btfsc	TMP1,7		;nastaveni pinu
	bsf	DB7
	btfsc	TMP1,6
	bsf	DB6
	btfsc	TMP1,5
	bsf	DB5
	btfsc	TMP1,4
	bsf	DB4
;       ------------------------
	bsf	E		;odeslani
	nop
	nop
	bcf	E
;       ------------------------
	return
;-------------------------------------------------------------------------------
;	Pripravi bity 0-3 a posle na displej (TMP1)
;-------------------------------
dis_wr2
	bcf	DB7		;vsechny piny do log 0
	bcf	DB6
	bcf	DB5
	bcf	DB4
;       ------------------------
	btfsc	TMP1,3		;nastaveni pinu
	bsf	DB7
	btfsc	TMP1,2
	bsf	DB6
	btfsc	TMP1,1
	bsf	DB5
	btfsc	TMP1,0
	bsf	DB4
;       ------------------------
	bsf	E		;odeslani
	nop
	nop
	bcf	E
;       ------------------------
	return
;-------------------------------------------------------------------------------
;	Cekani na uvolneni od displeje
;-------------------------------
dis_bf
	bsf	DB4_T		;datove piny vstupni
	bsf	DB5_T
	bsf	DB6_T
	bsf	DB7_T
;       ------------------------
;	bcf	STATUS,RP0	;banka 0
;-------------------------------
bufl1	bsf	RW		;RW = "1"
	bcf	RS		;RS = "0"
;       ------------------------
	bsf	E		;E = "1" (min. 220ns)
	nop	
	btfss	BFL	
	goto	bufl0
	bcf	E		;E = "0"
	nop
	nop
	bsf	E		;E = "1"
	nop
	nop
	bcf	E		;E = "0"
	goto	bufl1		;opakuj, dokud nebude DB7 log 0
;       ------------------------
bufl0	bcf	E		;E = "0"
	nop
	nop
	bsf	E		;E = "1"
	nop
	nop
	bcf	E		;E = "0"
;       ------------------------
	bcf	RW
;-------------------------------
;	bsf	STATUS,RP0	;banka 1
;       ------------------------
	bcf	DB4_T		;datove piny vystupni
	bcf	DB5_T
	bcf	DB6_T
	bcf	DB7_T
;       ------------------------
;	bcf	STATUS,RP0	;banka 0
;       ------------------------
	return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Zozrazeni HEX
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;	W(hex) -> displej(2 znaky)
;-------------------------------
disphe2
	movwf	TMP2
	movwf	TMP3
;	------------------------
	swapf	TMP2,1		;uprava registru desitky/jednotky
	movlw	0x0F
	andwf	TMP2,1
	andwf	TMP3,1
;	------------------------
	movf	TMP2,0
	call	disphex
	movf	TMP3,0
	call	disphex
;	------------------------
	return
;-------------------------------------------------------------------------------
;	W(hex) -> displej(1 znak)
;-------------------------------
disphex
	andlw	0x0F		;vymaz horni 4 bity
;	------------------------
	call	hetodis		;preved do znakove sady
	call	dis_zna		;zobraz na displeji
;	------------------------
	return
;-------------------------------
hetodis
	movwf	DAT
	btfss	STATUS,Z
	goto	hd1	
	retlw	0x4F		;0
;	------------------------
hd1	decfsz	DAT,1
	goto	hd2		
	retlw	0x31		;1
;	------------------------
hd2	decfsz	DAT,1
	goto	hd3		
	retlw	0x32		;2
;	------------------------
hd3	decfsz	DAT,1
	goto	hd4		
	retlw	0x33		;3
;	------------------------
hd4	decfsz	DAT,1
	goto	hd5		
	retlw	0x34		;4
;	------------------------
hd5	decfsz	DAT,1
	goto	hd6	
	retlw	0x35		;5
;	------------------------
hd6	decfsz	DAT,1
	goto	hd7	
	retlw	0x36		;6
;	------------------------
hd7	decfsz	DAT,1
	goto	hd8		
	retlw	0x37		;7
;	------------------------
hd8	decfsz	DAT,1
	goto	hd9		
	retlw	0x38		;8
;	------------------------
hd9	decfsz	DAT,1
	goto	hd10	
	retlw	0x39		;9
;	------------------------
hd10	decfsz	DAT,1
	goto	hd11		
	retlw	0x41		;A
;	------------------------
hd11	decfsz	DAT,1
	goto	hd12		
	retlw	0x42		;B
;	------------------------
hd12	decfsz	DAT,1
	goto	hd13		
	retlw	0x43		;C
;	------------------------
hd13	decfsz	DAT,1
	goto	hd14		
	retlw	0x44		;D
;	------------------------
hd14	decfsz	DAT,1
	goto	hd15		
	retlw	0x45		;E
;	------------------------
hd15	decfsz	DAT,1
	goto	hd16	
	retlw	0x46		;F
;	------------------------
hd16	return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Cekaci smycky
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;	Cekaci smycka 5ms
;-------------------------------
d_c5m
	movlw	.40		;(40)
	movwf	CEK2
d_c5m_2	movlw	.207		;(207)
	movwf	CEK1
;       ------------------------
d_c5m_1	decfsz	CEK1,1
	goto	d_c5m_1
	decfsz	CEK2,1
	goto	d_c5m_2
;       ------------------------
	return
;-------------------------------------------------------------------------------
;	Cekaci smycka 500ms
;-------------------------------
d_cs5
	movlw	.40  		;(40)
	movwf	CEK3
d_cs5_3	movlw	.126		;(126)
	movwf	CEK2
d_cs5_2	movlw	.164		;(164)
	movwf	CEK1
;       ------------------------
d_cs5_1	decfsz	CEK1,1
	goto	d_cs5_1
	decfsz	CEK2,1
	goto	d_cs5_2
	decfsz	CEK3,1
	goto	d_cs5_3
;       ------------------------
	return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;	Znakova sada Western-European (01)
;
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
znak_	movlw	0x20
	call	dis_zna
	return
;-------------------------------
znak_0	movlw	0x4F
	call	dis_zna
	return
;-------------------------------
znak_1	movlw	0x31
	call	dis_zna
	return
;-------------------------------
znak_2	movlw	0x32
	call	dis_zna
	return
;-------------------------------
znak_3	movlw	0x33
	call	dis_zna
	return
;-------------------------------
znak_4	movlw	0x34
	call	dis_zna
	return
;-------------------------------
znak_5	movlw	0x35
	call	dis_zna
	return
;-------------------------------
znak_6	movlw	0x36
	call	dis_zna
	return
;-------------------------------
znak_7	movlw	0x37
	call	dis_zna
	return
;-------------------------------
znak_8	movlw	0x38
	call	dis_zna
	return
;-------------------------------
znak_9	movlw	0x39
	call	dis_zna
	return
;-------------------------------
;-------------------------------
znak_A	movlw	0x41
	call	dis_zna
	return
;-------------------------------
znak_B	movlw	0x42
	call	dis_zna
	return
;-------------------------------
znak_C	movlw	0x43
	call	dis_zna
	return
;-------------------------------
znak_D	movlw	0x44
	call	dis_zna
	return
;-------------------------------
znak_E	movlw	0x45
	call	dis_zna
	return
;-------------------------------
znak_F	movlw	0x46
	call	dis_zna
	return
;-------------------------------
znak_G	movlw	0x47
	call	dis_zna
	return
;-------------------------------
znak_H	movlw	0x48
	call	dis_zna
	return
;-------------------------------
znak_I	movlw	0x49
	call	dis_zna
	return
;-------------------------------
znak_J	movlw	0x4A
	call	dis_zna
	return
;-------------------------------
znak_K	movlw	0x4B
	call	dis_zna
	return
;-------------------------------
znak_L	movlw	0x4C
	call	dis_zna
	return
;-------------------------------
znak_M	movlw	0x4D
	call	dis_zna
	return
;-------------------------------
znak_N	movlw	0x4E
	call	dis_zna
	return
;-------------------------------
znak_O	movlw	0x4F
	call	dis_zna
	return
;-------------------------------
znak_P	movlw	0x50
	call	dis_zna
	return
;-------------------------------
znak_Q	movlw	0x51
	call	dis_zna
	return
;-------------------------------
znak_R	movlw	0x52
	call	dis_zna
	return
;-------------------------------
znak_S	movlw	0x53
	call	dis_zna
	return
;-------------------------------
znak_T	movlw	0x54
	call	dis_zna
	return
;-------------------------------
znak_U	movlw	0x55
	call	dis_zna
	return
;-------------------------------
znak_V	movlw	0x56
	call	dis_zna
	return
;-------------------------------
znak_W	movlw	0x57
	call	dis_zna
	return
;-------------------------------
znak_X	movlw	0x58
	call	dis_zna
	return
;-------------------------------
znak_Y	movlw	0x59
	call	dis_zna
	return
;-------------------------------
znak_Z	movlw	0x5A
	call	dis_zna
	return
;-------------------------------
;-------------------------------
znak_a	movlw	0x61
	call	dis_zna
	return
;-------------------------------
znak_b	movlw	0x62
	call	dis_zna
	return
;-------------------------------
znak_c	movlw	0x63
	call	dis_zna
	return
;-------------------------------
znak_d	movlw	0x64
	call	dis_zna
	return
;-------------------------------
znak_e	movlw	0x65
	call	dis_zna
	return
;-------------------------------
znak_f	movlw	0x66
	call	dis_zna
	return
;-------------------------------
znak_g	movlw	0x67
	call	dis_zna
	return
;-------------------------------
znak_h	movlw	0x68
	call	dis_zna
	return
;-------------------------------
znak_i	movlw	0x69
	call	dis_zna
	return
;-------------------------------
znak_j	movlw	0x6A
	call	dis_zna
	return
;-------------------------------
znak_k	movlw	0x6B
	call	dis_zna
	return
;-------------------------------
znak_l	movlw	0x6C
	call	dis_zna
	return
;-------------------------------
znak_m	movlw	0x6D
	call	dis_zna
	return
;-------------------------------
znak_n	movlw	0x6E
	call	dis_zna
	return
;-------------------------------
znak_o	movlw	0x6F
	call	dis_zna
	return
;-------------------------------
znak_p	movlw	0x70
	call	dis_zna
	return
;-------------------------------
znak_q	movlw	0x71
	call	dis_zna
	return
;-------------------------------
znak_r	movlw	0x72
	call	dis_zna
	return
;-------------------------------
znak_s	movlw	0x73
	call	dis_zna
	return
;-------------------------------
znak_t	movlw	0x74
	call	dis_zna
	return
;-------------------------------
znak_u	movlw	0x75
	call	dis_zna
	return
;-------------------------------
znak_v	movlw	0x76
	call	dis_zna
	return
;-------------------------------
znak_w	movlw	0x77
	call	dis_zna
	return
;-------------------------------
znak_x	movlw	0x78
	call	dis_zna
	return
;-------------------------------
znak_y	movlw	0x79
	call	dis_zna
	return
;-------------------------------
znak_z	movlw	0x7A
	call	dis_zna
	return
;-------------------------------
;-------------------------------
znak_rh	movlw	0x00		; (r s hackem)
	call	dis_zna
	return
;-------------------------------
znak_yc	movlw	0x01		; (y s carkou)
	call	dis_zna
	return
;-------------------------------
znak_eh	movlw	0x02		; (e s hackem)
	call	dis_zna
	return
;-------------------------------
znak_sh	movlw	0x03		; (s s hackem)
	call	dis_zna
	return
;-------------------------------
znak_zh	movlw	0x04		; (z s hackem)
	call	dis_zna
	return
;-------------------------------
znak_uk	movlw	0x05		; (u s krouzkem)
	call	dis_zna
	return
;-------------------------------
znak_ic	movlw	0xA3		; (i s carkou)
	call	dis_zna
	return
;-------------------------------
znak_uc	movlw	0x86		; (u s carkou)
	call	dis_zna
	return
;-------------------------------
znak_oc	movlw	0x89		; (o s carkou)
	call	dis_zna
	return
;-------------------------------
znak_ac	movlw	0x9D		; (a s carkou)
	call	dis_zna
	return
;-------------------------------
znak_ec	movlw	0x96		; (e s carkou)
	call	dis_zna
	return
;-------------------------------
znak_ch	movlw	0x06		; (c s hackem)
	call	dis_zna
	return
;-------------------------------
;-------------------------------
znak_mi	movlw	0x2D		;-
	call	dis_zna
	return
;-------------------------------
znak_pl	movlw	0x2B		;+
	call	dis_zna
	return
;-------------------------------
znak_zm	movlw	0x3C		;<
	call	dis_zna
	return
;-------------------------------
znak_zv	movlw	0x3E		;>
	call	dis_zna
	return
;-------------------------------
znak_te	movlw	0x2E		;.
	call	dis_zna
	return
;-------------------------------
znak_ca	movlw	0x2C		;,
	call	dis_zna
	return
;-------------------------------
znak_st	movlw	0x07		;c
	call	dis_zna
	return
;-------------------------------
znak_dt	movlw	0x3A		;:
	call	dis_zna
	return
;-------------------------------
znak_vy	movlw	0x21		;!
	call	dis_zna
	return
;-------------------------------
znak_lo	movlw	0x2F		;/
	call	dis_zna
	return
;-------------------------------
znak_sp	movlw	0x7E		;sipka doprava
	call	dis_zna
	return
;-------------------------------
znak_sl	movlw	0x7F		;sipka doleva
	call	dis_zna
	return
;-------------------------------
znak_sn	movlw	0xB3		;sipka nahoru
	call	dis_zna
	return
;-------------------------------
znak_sd	movlw	0xB4		;sipka dolu
	call	dis_zna
	return
;-------------------------------
znak_pr	movlw	0x25		;%
	call	dis_zna
	return
;-------------------------------
znak_ro	movlw	0x3D		;=
	call	dis_zna
	return
;-------------------------------
znak_ot	movlw	0x3F		;?
	call	dis_zna
	return
;-------------------------------
znak_ff	movlw	0xFF		;plny stav
	call	dis_zna
	return